@@ -31,9 +31,10 @@ typedef struct {
 	GObjectClass parent_class;
 
 	/* signals */
-	void (*request_started) (SoupSession *, SoupMessage *, SoupSocket *);
-	void (*authenticate)    (SoupSession *, SoupMessage *,
-				 SoupAuth *, gboolean);
+	void (*request_started) (SoupSession *session, SoupMessage *msg,
+				 SoupSocket *socket);
+	void (*authenticate)    (SoupSession *session, SoupMessage *msg,
+				 SoupAuth *auth, gboolean retrying);
 
 	/* methods */
 	void  (*queue_message)   (SoupSession *session, SoupMessage *msg,
@@ -61,6 +62,7 @@ GType soup_session_get_type (void);
 #define SOUP_SESSION_SSL_CA_FILE        "ssl-ca-file"
 #define SOUP_SESSION_ASYNC_CONTEXT      "async-context"
 #define SOUP_SESSION_TIMEOUT		"timeout"
+#define SOUP_SESSION_USER_AGENT         "user-agent"
 
 GMainContext   *soup_session_get_async_context(SoupSession           *session);
 
