@@ -81,19 +81,15 @@ static void append_uri_encoded (GString *str, const char *in, const char *extra_
 static char *uri_decoded_copy (const char *str, int length);
 static char *uri_normalized_copy (const char *str, int length, const char *unescape_extra);
 
-static const char *http_scheme, *https_scheme;
+const char *_SOUP_URI_SCHEME_HTTP, *_SOUP_URI_SCHEME_HTTPS;
 
 static inline const char *
 soup_uri_get_scheme (const char *scheme, int len)
 {
 	if (len == 4 && !strncmp (scheme, "http", 4)) {
-		if (G_UNLIKELY (!http_scheme))
-			http_scheme = g_intern_static_string ("http");
-		return http_scheme;
+		return SOUP_URI_SCHEME_HTTP;
 	} else if (len == 5 && !strncmp (scheme, "https", 5)) {
-		if (G_UNLIKELY (!https_scheme))
-			https_scheme = g_intern_static_string ("https");
-		return https_scheme;
+		return SOUP_URI_SCHEME_HTTPS;
 	} else {
 		char *lower_scheme;
 
@@ -107,9 +103,9 @@ soup_uri_get_scheme (const char *scheme, int len)
 static inline guint
 soup_scheme_default_port (const char *scheme)
 {
-	if (scheme == http_scheme)
+	if (scheme == SOUP_URI_SCHEME_HTTP)
 		return 80;
-	else if (scheme == https_scheme)
+	else if (scheme == SOUP_URI_SCHEME_HTTPS)
 		return 443;
 	else
 		return 0;
@@ -339,7 +335,8 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 	}
 
 	/* HTTP-specific stuff */
-	if (uri->scheme == http_scheme || uri->scheme == https_scheme) {
+	if (uri->scheme == SOUP_URI_SCHEME_HTTP ||
+	    uri->scheme == SOUP_URI_SCHEME_HTTPS) {
 		if (!uri->host) {
 			soup_uri_free (uri);
 			return NULL;
@@ -754,8 +751,8 @@ soup_uri_normalize (const char *part, const char *unescape_extra)
 gboolean
 soup_uri_uses_default_port (SoupURI *uri)
 {
-	g_return_val_if_fail (uri->scheme == http_scheme ||
-			      uri->scheme == https_scheme, FALSE);
+	g_return_val_if_fail (uri->scheme == SOUP_URI_SCHEME_HTTP ||
+			      uri->scheme == SOUP_URI_SCHEME_HTTPS, FALSE);
 
 	return uri->port == soup_scheme_default_port (uri->scheme);
 }
@@ -883,8 +880,7 @@ soup_uri_set_query (SoupURI *uri, const char *query)
  * @form: a #GHashTable containing HTML form information
  *
  * Sets @uri's query to the result of encoding @form according to the
- * HTML form rules. See soup_form_encode_urlencoded() for more
- * information.
+ * HTML form rules. See soup_form_encode_hash() for more information.
  **/
 void
 soup_uri_set_query_from_form (SoupURI *uri, GHashTable *form)
@@ -893,6 +889,30 @@ soup_uri_set_query_from_form (SoupURI *uri, GHashTable *form)
 	uri->query = soup_form_encode_urlencoded (form);
 }
 
+/**
+ * soup_uri_set_query_from_fields:
+ * @uri: a #SoupURI
+ * @first_field: name of the first form field to encode into query
+ * @...: value of @first_field, followed by additional field names
+ * and values, terminated by %NULL.
+ *
+ * Sets @uri's query to the result of encoding the given form fields
+ * and values according to the * HTML form rules. See
+ * soup_form_encode() for more information.
+ **/
+void
+soup_uri_set_query_from_fields (SoupURI    *uri,
+				const char *first_field,
+				...)
+{
+	va_list args;
+
+	g_free (uri->query);
+	va_start (args, first_field);
+	uri->query = soup_form_encode_valist (first_field, args);
+	va_end (args);
+}
+
 /**
  * soup_uri_set_fragment:
  * @uri: a #SoupURI
@@ -911,13 +931,14 @@ soup_uri_set_fragment (SoupURI *uri, const char *fragment)
 GType
 soup_uri_get_type (void)
 {
-	static GType type = 0;
+	static volatile gsize type_volatile = 0;
 
-	if (G_UNLIKELY (type == 0)) {
-		type = g_boxed_type_register_static (
+	if (g_once_init_enter (&type_volatile)) {
+		GType type = g_boxed_type_register_static (
 			g_intern_static_string ("SoupURI"),
-			(GBoxedCopyFunc)soup_uri_copy,
-			(GBoxedFreeFunc)soup_uri_free);
+			(GBoxedCopyFunc) soup_uri_copy,
+			(GBoxedFreeFunc) soup_uri_free);
+		g_once_init_leave (&type_volatile, type);
 	}
-	return type;
+	return type_volatile;
 }
